@@ -2,19 +2,20 @@
 /*  GMime
  *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -28,8 +29,9 @@
 
 #include "internet-address.h"
 #include "gmime-table-private.h"
-#include "gmime-utils.h"
+#include "gmime-parse-utils.h"
 #include "gmime-iconv-utils.h"
+#include "gmime-utils.h"
 
 
 #ifdef ENABLE_WARNINGS
@@ -40,11 +42,6 @@
 
 #define d(x)
 
-/* public functions that we don't care for the user of GMime to know or care about */
-void decode_lwsp (const char **in);
-char *decode_word (const char **in);
-char *decode_addrspec (const char **in);
-
 
 /**
  * internet_address_new:
@@ -54,7 +51,7 @@ char *decode_addrspec (const char **in);
  * Returns a new #InternetAddress object.
  **/
 InternetAddress *
-internet_address_new ()
+internet_address_new (void)
 {
 	InternetAddress *ia;
 	
@@ -595,264 +592,6 @@ internet_address_list_to_string (const InternetAddressList *list, gboolean encod
 	return str;
 }
 
-
-void
-decode_lwsp (const char **in)
-{
-	const char *inptr = *in;
-	
-	while (*inptr && (*inptr == '(' || is_lwsp (*inptr))) {
-		while (*inptr && is_lwsp (*inptr))
-			inptr++;
-		
-		/* skip over any comments */
-		if (*inptr == '(') {
-			int depth = 1;
-			
-			inptr++;
-			while (*inptr && depth) {
-				if (*inptr == '\\' && *(inptr + 1))
-					inptr++;
-				else if (*inptr == '(')
-					depth++;
-				else if (*inptr == ')')
-					depth--;
-				
-				inptr++;
-			}
-		}
-	}
-	
-	*in = inptr;
-}
-
-static char *
-decode_quoted_string (const char **in)
-{
-	const char *inptr = *in;
-	char *out = NULL;
-	
-	decode_lwsp (&inptr);
-	if (*inptr == '"') {
-		out = (char *) inptr;
-		
-		inptr++;
-		while (*inptr && *inptr != '"') {
-			if (*inptr == '\\')
-				inptr++;
-			
-			if (*inptr)
-				inptr++;
-		}
-		
-		if (*inptr == '"')
-			inptr++;
-		
-		out = g_strndup (out, inptr - out);
-	}
-	
-	*in = inptr;
-	
-	return out;
-}
-
-static char *
-decode_atom (const char **in)
-{
-	const char *inptr = *in, *start;
-	
-	decode_lwsp (&inptr);
-	start = inptr;
-	while (is_atom (*inptr))
-		inptr++;
-	*in = inptr;
-	if (inptr > start)
-		return g_strndup (start, inptr - start);
-	else
-		return NULL;
-}
-
-char *
-decode_word (const char **in)
-{
-	const char *inptr = *in;
-	
-	decode_lwsp (&inptr);
-	if (*inptr == '"') {
-		*in = inptr;
-		return decode_quoted_string (in);
-	} else {
-		*in = inptr;
-		return decode_atom (in);
-	}
-}
-
-static gboolean
-decode_subliteral (const char **in, GString *domain)
-{
-	const char *inptr = *in;
-	gboolean got = FALSE;
-	
-	while (*inptr && *inptr != '.' && *inptr != ']') {
-		if (is_dtext (*inptr)) {
-			g_string_append_c (domain, *inptr);
-			inptr++;
-			got = TRUE;
-		} else if (is_lwsp (*inptr))
-			decode_lwsp (&inptr);
-		else
-			break;
-	}
-	
-	*in = inptr;
-	
-	return got;
-}
-
-static void
-decode_domain_literal (const char **in, GString *domain)
-{
-	const char *inptr = *in;
-	
-	decode_lwsp (&inptr);
-	while (*inptr && *inptr != ']') {
-		if (decode_subliteral (&inptr, domain) && *inptr == '.') {
-			g_string_append_c (domain, *inptr);
-			inptr++;
-		} else if (*inptr != ']') {
-			w(g_warning ("Malformed domain-literal, unexpected char '%c': %s",
-				     *inptr, *in));
-			
-			/* try and skip to the next char ?? */
-			inptr++;
-		}
-	}
-	
-	*in = inptr;
-}
-
-static char *
-decode_domain (const char **in)
-{
-	const char *inptr, *save;
-	GString *domain;
-	char *dom, *atom;
-	
-	domain = g_string_new ("");
-	
-	inptr = *in;
-	while (inptr && *inptr) {
-		decode_lwsp (&inptr);
-		if (*inptr == '[') {
-			/* domain literal */
-			g_string_append_c (domain, '[');
-			inptr++;
-			
-			decode_domain_literal (&inptr, domain);
-			
-			if (*inptr == ']') {
-				g_string_append_c (domain, ']');
-				inptr++;
-			} else
-				w(g_warning ("Missing ']' in domain-literal: %s", *in));
-		} else {
-			if (!(atom = decode_atom (&inptr))) {
-				w(g_warning ("Unexpected char '%c' in domain: %s", *inptr, *in));
-				/* remove the last '.' */
-				if (domain->len && domain->str[domain->len - 1] == '.')
-					g_string_truncate (domain, domain->len - 1);
-				break;
-			}
-			
-			g_string_append (domain, atom);
-			g_free (atom);
-		}
-		
-		save = inptr;
-		decode_lwsp (&inptr);
-		if (*inptr != '.') {
-			inptr = save;
-			break;
-		}
-		
-		g_string_append_c (domain, '.');
-		inptr++;
-	}
-	
-	if (domain->len)
-		dom = domain->str;
-	else
-		dom = NULL;
-	
-	g_string_free (domain, dom ? FALSE : TRUE);
-	
-	*in = inptr;
-	
-	return dom;
-}
-
-char *
-decode_addrspec (const char **in)
-{
-	char *domain, *word, *str = NULL;
-	const char *inptr;
-	GString *addrspec;
-	
-	decode_lwsp (in);
-	inptr = *in;
-	
-	if (!(word = decode_word (&inptr))) {
-		w(g_warning ("No local-part in addr-spec: %s", *in));
-		return NULL;
-	}
-	
-	addrspec = g_string_new (word);
-	g_free (word);
-	
-	/* get the rest of the local-part */
-	decode_lwsp (&inptr);
-	while (*inptr == '.') {
-		g_string_append_c (addrspec, *inptr++);
-		word = decode_word (&inptr);
-		if (word) {
-			g_string_append (addrspec, word);
-			decode_lwsp (&inptr);
-			g_free (word);
-		} else {
-			w(g_warning ("Invalid local-part in addr-spec: %s", *in));
-			goto exception;
-		}
-	}
-	
-	/* we should be at the '@' now... */
-	if (*inptr++ != '@') {
-		w(g_warning ("Invalid addr-spec; missing '@': %s", *in));
-		goto exception;
-	}
-	
-	if (!(domain = decode_domain (&inptr))) {
-		w(g_warning ("No domain in addr-spec: %s", *in));
-		goto exception;
-	}
-	
-	g_string_append_c (addrspec, '@');
-	g_string_append (addrspec, domain);
-	g_free (domain);
-	
-	str = addrspec->str;
-	g_string_free (addrspec, FALSE);
-	
-	*in = inptr;
-	
-	return str;
-	
- exception:
-	
-	g_string_free (addrspec, TRUE);
-	
-	return NULL;
-}
-
 static InternetAddress *
 decode_mailbox (const char **in)
 {
